//
//  GMRouterCase.m
//  GMRouter_Tests
//
//  Created by Q14 on 2020/8/13.
//  Copyright © 2020 Q14. All rights reserved.
//

#import <XCTest/XCTest.h>
#import "GMViewController.h"
#import <GMRouter/GMRouter.h>
#import <GMRouter/GMRouter+gm.h>
#import "GMRouterTool.h"

@interface Target_GMRouterCase : XCTestCase

@end

@implementation Target_GMRouterCase

- (void)setUp {
    // Put setup code here. This method is called before the invocation of each test method in the class.
    
    NSMutableArray *targets = [NSMutableArray arrayWithArray:[GMRouter sharedInstance].targets];
    [targets addObject:@"Target_GMRouterCase"];
    [GMRouter sharedInstance].targets = targets;
    
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
}

- (void)testExample {
    // This is an example of a functional test case.
    // Use XCTAssert and related functions to verify your tests produce the correct results.
    //测试不传参数的case
    [self testNoParamsCase];
    
     //测试传参数的case
    [self testParamsCase];
    //测试回调
    [self testParams_action];
    
    //以下主要测试 pushUrlScheme parmas 这个方法的几种情况
    [self testJustScheme];
    [self testSchemeAndDict];
    [self testQuerySchemeAndDict];
}



#pragma mark - test 裸协议 just_scheme
- (void)testJustScheme {
    [[GMRouter sharedInstance] pushScheme:@"just_scheme" params:NULL];
}

- (UIViewController *)just_scheme:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    XCTAssertTrue(params.allKeys.count == 0, @"just_scheme 应该不会有参数");
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}

#pragma mark - test scheme_params 以及字典参数
- (void)testSchemeAndDict {
    [[GMRouter sharedInstance] pushScheme:@"scheme_params" params:@{@"service_id":@"3333"}];
}

- (UIViewController *)scheme_params:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    XCTAssertTrue(params.allKeys.count != 0, @"scheme_params 应该有参数");
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}

#pragma mark - test query_scheme_params 以及字典参数
- (void)testQuerySchemeAndDict {
    [[GMRouter sharedInstance] pushScheme:@"query_scheme_params?title=无极" params:@{@"service_id": @"3333"}];
}

- (UIViewController *)query_scheme_params:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    XCTAssertTrue(params.allKeys.count == 2, @"query_scheme_params 应该有两个参数");
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}




- (void)testNoParamsCase {
    [[GMRouter sharedInstance] pushScheme:@"common_no_params"];
    [[GMRouter sharedInstance] pushScheme:@"gengmei://common_no_params"];
}

- (void)testParamsCase {
    [[GMRouter sharedInstance] pushScheme:@"common_params?title=无极&name=杨二&age=10"];
    [[GMRouter sharedInstance] pushScheme:@"gengmei://common_params?title=无极&name=杨二&age=10"];
}

- (void)testParams_action {
    
   UIViewController *vc =  (UIViewController *)[[GMRouter sharedInstance] pushScheme:@"common_params_action" params:@{
        @"title": @"无极",
        @"age":@10
    } completeBlock:^(NSDictionary * _Nonnull params) {
        NSLog(@"收到回调");
    }];
   if (!vc.routerCompleteBlock) {
        XCTAssertTrue(NO, @"common_params_action block 应该有值");
    } else {
        vc.routerCompleteBlock(@{ @"title": @"无极", @"age":@10 });
    }
    
    UIViewController *vc2 =  (UIViewController *)[[GMRouter sharedInstance] pushScheme:@"common_params_action" params:@{
        @"title": @"无极",
        @"age":@10
    } completeBlock:^(NSDictionary * _Nonnull params) {
        NSLog(@"收到回调");
    }];
    if (!vc2.routerCompleteBlock) {
        XCTAssertTrue(NO, @"common_params_action block 应该有值");
    } else {
        vc2.routerCompleteBlock(@{ @"title": @"无极", @"age":@10 });
    }
}



- (void)testPerformanceExample {
    // This is an example of a performance test case.
    [self measureBlock:^{
        // Put the code you want to measure the time of here.
    }];
}

#pragma mark - 自定义外界传入urlscheme case 需要不断维护
- (UIViewController *)common_params:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    XCTAssertTrue(params.allKeys.count != 0, @"common_params 参数解析失败");
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}

- (UIViewController *)common_no_params:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    XCTAssertTrue(params.allKeys.count == 0, @"common_no_params 应该不会有参数");
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}

- (UIViewController *)common_params_action:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    if (vc.routerCompleteBlock) {
        XCTAssertTrue(NO, @"common_params_action 测试失败block 不应该有值");
    }
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}


- (UIViewController *)query_scheme:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    if (vc.routerCompleteBlock) {
        XCTAssertTrue(NO, @"common_params_action 测试失败block 不应该有值");
    }
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}

- (UIViewController *)scheme_custom_params:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    if (vc.routerCompleteBlock) {
        XCTAssertTrue(NO, @"common_params_action 测试失败block 不应该有值");
    }
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}

- (UIViewController *)query_scheme_custom_params:(NSDictionary *)params {
    GMViewController *vc = [[GMViewController alloc] init];
    if (vc.routerCompleteBlock) {
        XCTAssertTrue(NO, @"common_params_action 测试失败block 不应该有值");
    }
    [[GMRouterTool getCurrentViewController].navigationController pushViewController:vc animated:YES];
    return vc;
}



@end
